#include <imsls.h>
#include <stdlib.h>
#include <stdio.h>

int main()
{
  float *parameters = NULL, *outlier_forecast = NULL;
  int *outlier_stat = NULL;
  int n_obs, n_predict, i, num_outliers;
  float aic, res_sigma;
  int model[4];
  float forecast_table[24];

  float x[141] = {
    12.8,12.2,11.9,10.9,10.6,11.3,11.1,10.4,10.0,9.7,9.7,9.7,
    11.1,10.5,10.3,9.8,9.8,10.4,10.4,10.0,9.7,9.3,9.6,9.7,
    10.8,10.7,10.3,9.7,9.5,10.0,10.0,9.3,9.0,8.8,8.9,9.2,
    10.4,10.0,9.6,9.0,8.5,9.2,9.0,8.6,8.3,7.9,8.0,8.2,
    9.3,8.9,8.9,7.7,7.6,8.4,8.5,7.8,7.6,7.3,7.2,7.3,
    8.5,8.2,7.9,7.4,7.1,7.9,7.7,7.2,7.0,6.7,6.8,6.9,
    7.8,7.6,7.4,6.6,6.8,7.2,7.2,7.0,6.6,6.3,6.8,6.7,
    8.1,7.9,7.6,7.1,7.2,8.2,8.1,8.1,8.2,8.7,9.0,9.3,
    10.5,10.1,9.9,9.4,9.2,9.8,9.9,9.5,9.0,9.0,9.4,9.6,
    11.0,10.8,10.4,9.8,9.7,10.6,10.5,10.0,9.8,9.5,9.7,9.6,
    10.9,10.3,10.4,9.3,9.3,9.8,9.8,9.3,8.9,9.1,9.1,9.1,
    10.2,9.9,9.4,8.7,8.6,9.3,9.1,8.8,8.5};

  int times[141] = {
       1,2,3,4,5,6,7,8,9,10,11,12,
      13,14,15,16,17,18,19,20,21,22,23,24,
      25,26,27,28,29,30,31,32,33,34,35,36,
      37,38,39,40,41,42,43,44,45,46,47,48,
      49,50,51,52,53,54,55,56,57,58,59,60,
      61,62,63,64,65,66,67,68,69,70,71,72,
      73,74,75,76,77,78,79,80,81,82,83,84,
      85,86,87,88,89,90,91,92,93,94,95,96,
      97,98,99,100,101,102,103,104,105,106,107,108,
     109,110,111,112,113,114,115,116,117,118,119,120,
     121,122,123,124,125,126,127,128,129,130,131,132,
     133,134,135,136,137,138,139,140,141};

   n_predict = 6;
   n_obs = 135;

   parameters = imsls_f_auto_arima(n_obs, times, x, IMSLS_MODEL, model,
                       IMSLS_AIC, &aic,
                       IMSLS_MAX_LAG, 5,
                       IMSLS_CRITICAL, 4.0,
                       IMSLS_NUM_OUTLIERS, &num_outliers,
                       IMSLS_OUTLIER_STATISTICS, &outlier_stat,
                       IMSLS_RESIDUAL_SIGMA, &res_sigma,
                       IMSLS_NUM_PREDICT, n_predict,
                       IMSLS_OUTLIER_FORECAST, &outlier_forecast,
                       0);
                       
   printf("\nMethod 1: Automatic ARIMA model selection,"
		         " no differencing\n");
   printf("\nModel chosen: p=%d, q=%d, s=%d, d=%d\n", model[0],
	             model[1], model[2], model[3]);
   printf("\nNumber of outliers: %d\n\n", num_outliers);

   printf("Outlier statistics:\n\n");
   printf("Time point\t\tOutlier type\n");
   for (i=0; i<num_outliers; i++)
     printf("%d\t\t%d\n", outlier_stat[2*i], outlier_stat[2*i+1]);

   printf("\nAIC = %lf\n", aic);
   printf("RSE = %lf\n\n", res_sigma);

   printf("Parameters:\n");
   for (i=0; i<=model[0]+model[1]; i++)
     printf("parameters[%d]=%lf\n", i,  parameters[i]);

   for (i=0; i<n_predict; i++)
   {
      forecast_table[4*i] = x[n_obs+i];
      forecast_table[4*i+1] = outlier_forecast[3*i];
      forecast_table[4*i+2] = outlier_forecast[3*i+1];
      forecast_table[4*i+3] = outlier_forecast[3*i+2];
   }

   imsls_f_write_matrix("\t* * * Forecast Table * * *"
      "\nOrig. series\t  forecast\tprob. limits\tpsi weights\n",
      n_predict, 4, forecast_table, IMSLS_WRITE_FORMAT, "%11.4f", 0);

   if (parameters)
   {
      imsls_free(parameters);
      parameters = NULL;
   }

   if (outlier_forecast)
   {
      imsls_free(outlier_forecast);
      outlier_forecast = NULL;
   }

   if (outlier_stat)
   {
      imsls_free(outlier_stat);
      outlier_stat = NULL;
   }

   return;
}
